\name{text}
\title{Add Text to a Plot}
\usage{
text (x, \dots)
text.default (x, y = NULL, labels = seq(along = x), adj = NULL,
              pos = NULL, offset = 0.5, vfont = NULL, \dots)
}
\alias{text}
\alias{text.default}
\arguments{
  \item{x, y}{numeric vectors of coordinates where the text
    \code{labels} should be written.  If the length of \code{x} and
    \code{y} differs, the shorter one is recycled.}
  \item{labels}{one or more character strings or expressions specifying
    the \emph{text} to be written.}
  \item{adj}{one or two values in \eqn{[0,1]} which specify the x
    (and optionally y) adjustment of the labels.}
  \item{pos}{a position specifier for the text.  If specified this
    overrides any \code{adj} value given.  Values of \code{1},
    \code{2}, \code{3} and \code{4}, respectively indicate
    positions below, to the left of, above and to the right of
    the specified coordinates.}
  \item{offset}{when \code{pos} is specified, this value gives the
    offset of the label from the specified coordinate in fractions
    of a character width.}
  \item{vfont}{if a character vector of length 2 is specified, then Hershey
    vector fonts are used.  The first element of the vector
    selects a typeface and the second element selects a style 
    (see the Details section).} 
  \item{\dots}{further graphical parameters (from \code{\link{par}}),
    such as \code{cex}, etc.}
}
\description{
  \code{text} draws the strings given in the vector \code{labels} at the
  coordinates given by \code{x} and \code{y}.
  \code{y} may be missing since \code{\link{xy.coords}(x,y)} is used for
  construction of the coordinates.
}
\details{
  \code{labels} must be of type \code{\link{character}} or
  \code{\link{expression}}.  In the latter case, quite a bit of
  mathematical notation is available such as sub- and superscripts,
  greek letters, fractions, etc.

  \code{adj} allows \emph{adj}ustment of the text with respect to \code{(x,y)}.
  Values of 0, 0.5, and 1 specify left/bottom, middle and
  right/top, respectively.  The default is for centered text, i.e.,
  \code{adj = c(0.5, 0.5)}. Accurate vertical centering needs
  character metric information on individual characters, which is
  only available on some devices.

  The \code{pos} and \code{offset} arguments can be used in conjunction
  with values returned by \code{identify} to recreate an interactively
  labelled plot.

  Text can be rotated by using graphical parameters \code{srt} (see
  \code{\link{par}}); this rotates about the centre set by \code{adj}.

  The available values for \code{(typeface, fontindex)} for Hershey
  vector fonts are:
    \code{("serif", "symbol")},
    \code{("serif", "plain")},
    \code{("serif", "italic")},
    \code{("serif", "bold")},
    \code{("serif", "bold italic")},
    \code{("sans serif", "symbol")},
    \code{("sans serif", "plain")},
    \code{("sans serif", "italic")},
    \code{("sans serif", "bold")},
    \code{("sans serif", "bold italic")},
    \code{("script", "symbol")},
    \code{("script", "plain")},
    \code{("script", "italic")},
    \code{("script", "bold")},
    \code{("gothic english", "symbol")},
    \code{("gothic english", "plain")},
    \code{("gothic german", "symbol")},
    \code{("gothic german", "plain")},
    \code{("gothic italian", "symbol")},
    \code{("gothic italian", "plain")},
    \code{("serif symbol", "symbol")},
    \code{("serif symbol", "plain")},
    \code{("serif symbol", "italic")},
    \code{("serif symbol", "bold")},
    \code{("serif symbol", "bold italic")},
    \code{("sans serif symbol", "symbol")},
    \code{("sans serif symbol", "plain")},
    \code{("sans serif symbol", "italic")}.
}
\seealso{
  \code{\link{mtext}}, \code{\link{title}};
  \code{\link{plotmath}} for details and more examples on
  mathematical annotation.
}
\examples{
plot(-1:1,-1:1, type = "n", xlab = "Re", ylab = "Im")
K <- 16; text(exp(1i * 2 * pi * (1:K) / K), col = 2)

## The following two examples use latin1 characters: these may not
## appear correctly (or be omitted entirely).
plot(1:10, 1:10, main = "text(..) examples\n~~~~~~~~~~~~~~",
     sub = "R is GNU , but not  ..")
mtext("ISO-accents:    < <", side=3)
points(c(6,2), c(2,1), pch = 3, cex = 4, col = "red")
text(6, 2, "the text is CENTERED around (x,y) = (6,2) by default",
     cex = .8)
text(2, 1, "or Left/Bottom - JUSTIFIED at (2,1) by `adj = c(0,0)'",
     adj = c(0,0))
text(4, 9, expression(hat(beta) == (X^t * X)^{-1} * X^t * y))
text(4, 8.4, "expression(hat(beta) == (X^t * X)^{-1} * X^t * y)", cex = .75)
text(4, 7, expression(bar(x) == sum(frac(x[i], n), i==1, n)))

## Two more latin1 examples
text(5,10.2,"Le franais, c'est faile: Rgles, Libert, Egalit, Fraternit..")
text(5,9.8, "Jetz no chli zrittsch: (noch ein bichen Zrcher deutsch)")
}
\keyword{aplot}
